// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
//
// Do not check ref, because timings cannot be reproduced exactly.
// <-- NO CHECK REF -->
//

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//
function flag = assert_equal ( computed , expected )
  if ( and ( computed==expected ) ) then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

/////////////////////////////////////////////////////////////
// Pascal up matrix.
// Column by column version
function c = pascalup_col0rhs ()
   n = 100
   c = eye(n,n)
   c(1,:) = ones(1,n)
   for i = 2:(n-1)
      c(2:i,i+1) = c(1:(i-1),i)+c(2:i,i)
   end
endfunction

//
[t,msg] = scibench_benchfun ( pascalup_col0rhs );
assert_equal ( size(t) , [10 1] );
assert_equal ( typeof(t) , "constant" );
assert_equal ( and(t>=0) , %t );
assert_equal ( size(msg) , [1 1] );
assert_equal ( typeof(msg) , "string" );
//
[t,msg] = scibench_benchfun ( pascalup_col0rhs , "pascalup_col" );
assert_equal ( size(t) , [10 1] );
assert_equal ( typeof(t) , "constant" );
assert_equal ( and(t>=0) , %t );
assert_equal ( size(msg) , [1 1] );
assert_equal ( typeof(msg) , "string" );
//
// With an extra parameter
function c = pascalup_col (n)
   [lhs, rhs] = argn()
   if ( rhs <> 1 ) then
     error("Wrong number of input arguments")
   end
   c = eye(n,n)
   c(1,:) = ones(1,n)
   for i = 2:(n-1)
      c(2:i,i+1) = c(1:(i-1),i)+c(2:i,i)
   end
endfunction

[t,msg] = scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" );
assert_equal ( size(t) , [10 1] );
assert_equal ( typeof(t) , "constant" );
assert_equal ( and(t>=0) , %t );
assert_equal ( size(msg) , [1 1] );
assert_equal ( typeof(msg) , "string" );
//
[t,msg] = scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" , 1 );
assert_equal ( size(t) , [10 1] );
assert_equal ( typeof(t) , "constant" );
assert_equal ( and(t>=0) , %t );
assert_equal ( size(msg) , [1 1] );
assert_equal ( typeof(msg) , "string" );
//
[t,msg] = scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" , 1 , 10 );
assert_equal ( size(t) , [10 1] );
assert_equal ( typeof(t) , "constant" );
assert_equal ( and(t>=0) , %t );
assert_equal ( size(msg) , [1 1] );
assert_equal ( typeof(msg) , "string" );
//
[t,msg] = scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" , 1 , 10 , %t );
assert_equal ( size(t) , [10 1] );
assert_equal ( typeof(t) , "constant" );
assert_equal ( and(t>=0) , %t );
assert_equal ( size(msg) , [1 1] );
assert_equal ( typeof(msg) , "string" );

// Disable the message
clear t
clear msg
[t,msg] = scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" , 1 , 10 , %f );


///////////////////////////////////////////////////////////////
//
// With 2 input arguments
function F = subsets ( E , m )
   // Authors : Chancelier, Pincon
   // compute all the subsets with m elements of the set E
   // (E must be given as a row vector and all its elements
   // must be differents). For sets of numbers.
   [lhs, rhs] = argn()
   if ( rhs <> 2 ) then
     error("Wrong number of input arguments")
   end
   n = length(E)
   if m > n then
      F = []
   elseif m == n
      F = E
   elseif m == 1
      F = E';
   else
      F = [];
      for i = 1:n-m+1
         EE = E(i+1:n)
         FF = subsets(EE,m-1)
         mm = size(FF,1)
         F = [F ; E(i)*ones(mm,1),FF]
      end
   end
endfunction
scibench_benchfun ( list(subsets,1:10,3) , "subsets" , 1 , 10 , %t );
//
// With a primitive
scibench_benchfun ( list(sin,ones(100,100)) , "sin" , 1 , 10 , %t );
//
// With an additionnal output argument
function [c,n] = pascalup_col3 ( n )
   [lhs, rhs] = argn()
   if ( lhs <> 2 ) then
     error("Wrong number of output arguments")
   end
   c = eye(n,n)
   c(1,:) = ones(1,n)
   for i = 2:(n-1)
      c(2:i,i+1) = c(1:(i-1),i)+c(2:i,i)
   end
endfunction
scibench_benchfun ( list(pascalup_col3,100) , "pascalup_col" , 2 );
//
// Use a callback to print the timings
function stop = myoutfun(k,t)
  global __counter__
  // Do nothing
  __counter__ = __counter__ + 1
  stop = %f
  //mprintf("Run #%d, t=%.2f\n",k,t)
endfunction
global __counter__
__counter__ = 0;
scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" , [] , [] , [] , myoutfun );
assert_equal ( __counter__>0 , %t );
//
// Use a callback to print the timings.
// The callback requires additionnal arguments.
function stop = myoutfun2(k,t,data)
  global __counter__
  // Do nothing
  __counter__ = __counter__ + 1
  stop = %f
  if ( data <> 2 ) then
    error ( "Unknown data" )
  end
  //mprintf("Run #%d, t=%.2f\n",k,t)
endfunction
global __counter__
__counter__ = 0;
scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" , [] , [] , [] , list(myoutfun2,2) );
assert_equal ( __counter__>0 , %t );

//
// Use a callback to print the timings.
// The callback stops the algorithm
function stop = myoutfun3(k,t)
  global __counter__
  // Do nothing
  __counter__ = __counter__ + 1
  stop = ( __counter__ >= 2 )
  //mprintf("Run #%d, t=%.2f\n",k,t)
endfunction
global __counter__
__counter__ = 0;
scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" , [] , [] , [] , myoutfun3 );
assert_equal ( __counter__ >=2 , %t );

//
// Use a callback to print the timings and stop the algorithm.
function stop = myoutfun4(k,t,kmax)
  global __counter__
  // Do nothing
  __counter__ = __counter__ + 1
  stop = ( k >= kmax )
  //mprintf("Run #%d, t=%.2f\n",k,t)
endfunction
global __counter__
__counter__ = 0;
scibench_benchfun ( list(pascalup_col,100) , "pascalup_col" , [] , [] , [] , list(myoutfun4,2) );
assert_equal ( __counter__ >=2 , %t );

