// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
//
// Do not check ref, because timings cannot be reproduced exactly.
// <-- NO CHECK REF -->
//

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//
function flag = assert_equal ( computed , expected )
  if ( and ( computed==expected ) ) then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

/////////////////////////////////////////////////////////////
function c = pascalup_col (n)
   // Pascal up matrix.
   // Column by column version
   c = eye(n,n)
   c(1,:) = ones(1,n)
   for i = 2:(n-1)
      c(2:i,i+1) = c(1:(i-1),i)+c(2:i,i)
   end
endfunction

stacksize("max");

perftable = scibench_dynbenchfun ( pascalup_col );
assert_equal ( typeof(perftable) , "constant" );
assert_equal ( and(perftable(:,1)>=1) , %t );
assert_equal ( and(perftable(:,2)>=0) , %t );
//
perftable = scibench_dynbenchfun ( pascalup_col , [] , %f );
assert_equal ( typeof(perftable) , "constant" );
assert_equal ( and(perftable(:,1)>=1) , %t );
assert_equal ( and(perftable(:,2)>=0) , %t );
//
perftable = scibench_dynbenchfun ( pascalup_col , [] , %f , %f );
assert_equal ( typeof(perftable) , "constant" );
assert_equal ( and(perftable(:,1)>=1) , %t );
assert_equal ( and(perftable(:,2)>=0) , %t );
//
perftable = scibench_dynbenchfun ( pascalup_col , [] , %f , %f , 0.001 );
assert_equal ( typeof(perftable) , "constant" );
assert_equal ( and(perftable(:,1)>=1) , %t );
assert_equal ( and(perftable(:,2)>=0) , %t );
//
perftable = scibench_dynbenchfun ( pascalup_col , [] , %f , %f , 0.001 , 0.1 );
assert_equal ( typeof(perftable) , "constant" );
assert_equal ( and(perftable(:,1)>=1) , %t );
assert_equal ( and(perftable(:,2)>=0) , %t );
//
perftable = scibench_dynbenchfun ( pascalup_col , [] , %f , %f , 0.001 , 0.1 , 1.1 );
assert_equal ( typeof(perftable) , "constant" );
assert_equal ( and(perftable(:,1)>=1) , %t );
assert_equal ( and(perftable(:,2)>=0) , %t );


// With an additionnal input argument
function c = pascalup_col2 ( n , alpha )
   [lhs, rhs] = argn()
   if ( rhs <> 2 ) then
     error("Wrong number of input arguments")
   end
   c = eye(n,n) * alpha
   c(1,:) = ones(1,n)
   for i = 2:(n-1)
      c(2:i,i+1) = c(1:(i-1),i)+c(2:i,i)
   end
endfunction

scibench_dynbenchfun ( list(pascalup_col2,2) , [] , %f , %f , [] , 0.1 );
//
// With an additionnal output argument
function [c,n] = pascalup_col3 ( n )
   [lhs, rhs] = argn()
   if ( lhs <> 2 ) then
     error("Wrong number of output arguments")
   end
   c = eye(n,n)
   c(1,:) = ones(1,n)
   for i = 2:(n-1)
      c(2:i,i+1) = c(1:(i-1),i)+c(2:i,i)
   end
endfunction

scibench_dynbenchfun ( pascalup_col3 , 2 , %f , %f , [] , 0.1 );
//
// Use a callback to print the timings
function stop = myoutfun(k,n,t)
  global __counter__
  // Do nothing
  __counter__ = __counter__ + 1
  stop = %f
  //mprintf("Run #%d, n=%d, t=%.2f\n",k,n,t)
endfunction
global __counter__
__counter__ = 0;
scibench_dynbenchfun ( pascalup_col , [] , %f , %f , [] , 0.1 , [] , myoutfun );
assert_equal ( __counter__>0 , %t );

//
// Use a callback to print the timings.
// The callback requires additionnal arguments.
function stop = myoutfun2(k,n,t,data)
  global __counter__
  // Do nothing
  __counter__ = __counter__ + 1
  stop = %f
  //mprintf("Run #%d, n=%d, t=%.2f\n",k,n,t)
endfunction
global __counter__
__counter__ = 0;
scibench_dynbenchfun ( pascalup_col , [] , %f , %f , [] , 0.1 , [] , list(myoutfun2,2) );
assert_equal ( __counter__>0 , %t );

//
// Use a callback to print the timings.
// The callback stops the algorithm
function stop = myoutfun3(k,n,t)
  global __counter__
  // Do nothing
  __counter__ = __counter__ + 1
  stop = ( __counter__ >= 2 )
  //mprintf("Run #%d, n=%d, t=%.2f\n",k,n,t)
endfunction
global __counter__
__counter__ = 0;
scibench_dynbenchfun ( pascalup_col , [] , %f , %f , [] , 0.1 , [] , myoutfun3 );
assert_equal ( __counter__ >=2 , %t );

//
// Use a callback to print the timings and stop the algorithm.
function stop = myoutfun4(k,n,t,kmax)
  global __counter__
  // Do nothing
  __counter__ = __counter__ + 1
  stop = ( k >= kmax )
  //mprintf("Run #%d, n=%d, t=%.2f\n",k,n,t)
endfunction
global __counter__
__counter__ = 0;
scibench_dynbenchfun ( pascalup_col , [] , %f , %f , [] , 0.1 , [] , list(myoutfun4,2) );
assert_equal ( __counter__ >=2 , %t );


